@@ -1,7 +1,7 @@
 /* dns_query.h - Execute outgoing dns queries and write entries to cache
    Copyright (C) 2000, 2001 Thomas Moestl
 
-   With modifications by Paul Rombouts, 2002, 2003.
+   With modifications by Paul Rombouts, 2002, 2003, 2004.
 
 This file is part of the pdnsd package.
 
@@ -48,53 +48,49 @@ typedef struct {
 		struct sockaddr_in6 sin6;
 #endif
 	}                   a;
-	struct sockaddr     *sin;
-	long                timeout;
-	int                 flags;
-	int                 nocache;
-	int                 state;
-	int                 event;         /* event to poll for */
-	int                 nstate;
-	int                 qm;
+	time_t              timeout;
+	unsigned short      flags;
+	short               nocache;
+	short               state;
+	short               qm;
 	char                trusted;
         char                auth_serv;
-	unsigned char       nsdomain[256];
+	char                lean_query;
+	char                needs_testing;
+	unsigned char       *nsdomain;
 	/* internal state for p_exec_query */
 	int                 sock;
 /*	dns_cent_t          nent;
 	dns_cent_t          servent;*/
 	unsigned short      transl;
 	unsigned short      recvl;
+#ifndef NO_TCP_QUERIES
+	int                 iolen;  /* number of bytes written or read up to now */
+#endif
 	dns_hdr_t           *hdr;
-	int                 myrid;
 	dns_hdr_t           *recvbuf;
-	int                 qt;
-	char                lean_query;
+	unsigned short      myrid;
+	unsigned short      qt;
 	int                 s_errno;
 } query_stat_t;
 typedef DYNAMIC_ARRAY(query_stat_t) *query_stat_array;
 
 #define QS_INITIAL       0  /* This is the initial state. Set this before starting. */
-#define QS_QUERY         1
-#define QS_DONE         11  /* done, resources freed, result is in stat_t */
 
+#define QS_TCPINITIAL    1  /* Start a TCP query. */
+#define QS_TCPWRITE      2  /* Waiting to write data. */
+#define QS_TCPREAD       3  /* Waiting to read data. */
 
-#define QSN_TCPINITIAL   1  /* Start a TCP query. */
-#define QSN_TCPALLOC     2  /* Resources allocated */
-#define QSN_TCPCONNECT   3  /* Connected. */
-#define QSN_TCPLWRITTEN  4  /* Query length has been transmitted. */
-#define QSN_TCPQWRITTEN  5  /* Query transmitted. */
-#define QSN_TCPLREAD     6  /* Answer length read */
+#define QS_UDPINITIAL    4  /* Start a UDP query */
+#define QS_UDPRECEIVE    5  /* UDP query transmitted, waiting for response. */
 
-#define QSN_UDPINITIAL  20  /* Start a UDP query */
-#define QSN_UDPTRANSMIT 21  /* Start a UDP query */
-#define QSN_UDPRECEIVE  22  /* Start a UDP query */
+#define QS_QUERY_CASES   case QS_TCPINITIAL: case QS_TCPWRITE: case QS_TCPREAD: case QS_UDPINITIAL: case QS_UDPRECEIVE
+#define QS_DONE          8  /* done, resources freed, result is in stat_t */
 
-#define QSN_DONE        11
 
 /* Events to be polled/selected for */
-#define QEV_WRITE        1
-#define QEV_READ         2
+#define QS_WRITE_CASES case QS_TCPWRITE
+#define QS_READ_CASES  case QS_TCPREAD: case QS_UDPRECEIVE
 
 /* --- parallel query */
 int p_dns_cached_resolve(query_stat_array q, unsigned char *name, unsigned char *rrn , dns_cent_t **cached, int hops, int thint, time_t queryts);
